/*
  La classe JnmapThread implementa il Thread che getisce il processo nmap. Tale classe  stata implementata per la
  necessit di svincolare la classe che definisce gli elementi grafici dell'interfaccia, dalla gestione specifica
  del processo. Nmap, lavorando in rete, pu necessitare di un tempo sensibilmente lungo per portare a termine la
  propria elaborazione. Se i risultati sono analizzati dalla classe Jnmap si rischia un crash dell'interfaccia e si
  rendono inacessibili altri elementi grafici (bloccati a causa dell'attesa della terminazione del processo).
 
  La classe  stata scritta da Dario Maggiari.
 */

import java.io.*;
import java.util.*;
import javax.swing.text.*;
import javax.swing.*;

public class JnmapThread extends Thread
  {
    private String cmd;
    private String stat;
    private Process process_nmap;
    private Runtime run;
    private Jnmap jmp;
    private JTextArea text_er;
    private boolean secure = false;
    private int leng = 10140;
    
    /**
      Costruttore della classe.
      @param command Il comando (nmap + parametri) da lanciare.
      @param running Un oggetto Runtime per la gestione dei processi.
      @param jmpi L'interfaccia stessa con cui il thread deve dialogare. 
     */
    public JnmapThread(String command, Runtime running, Jnmap jmpi) 
      {
       this.cmd = command;   
       this.run = running;
       this.jmp = jmpi;
      }
    
    /**
      Il metodo run implementa la gestione del processo nmap.
      Il Thread JnmapThread si fa carico di gestire il processo nmap e di dialogare con l'interfaccia per 
      il setting di alcuni elementi grafici al verificarsi di determinate situazioni: terminazione regolare del 
      processo, errore nel lancio del processo, terminazione del processo forzata dall'utente.
     */
    public void run()
      {
       try
          { 
           byte[] b_streamok = new byte[leng];
           byte[] b_streamer = new byte[1024];   
       
           process_nmap = run.exec(cmd);
           int vq = process_nmap.waitFor(); //Attende la terminazione dell'esecuzione del programma. 
           
           JLabel status_scan = jmp.getStatusScan();
           JLabel iconscan = jmp.getIconScan();
           JLabel timeinfo = jmp.getTimeInfos();
           JLabel labelip = jmp.getLabelFocus();
           boolean ilstate = jmp.getiLState();
           
           JButton b_kill = jmp.getBSKill();
           JButton b_scan = jmp.getBScan();
           
           JTabbedPane Tabbed = jmp.getTabbed();
           JTextArea textresult = jmp.getTextResult();
           text_er = jmp.getAreaError();  
           JTextField textip = jmp.getTextIp();
           
           if(ilstate) { stat = "Target(s) from File"; }
           
           else { stat = textip.getText(); }

           if(vq == 1) //Errore nell'esecuzione di nmap.
              {          
               Tabbed.setEnabled(true);
               iconscan.setIcon(new ImageIcon(getClass().getResource("/img/noscan.gif"))); //Ripristina gif statica.
                
               JLabel lab_host_t = jmp.getHostT();
               lab_host_t.setText("No Target.");
               JLabel lab_cmd = jmp.getCNmap();
               lab_cmd.setText(cmd);
               status_scan.setText("Error launching Nmap!");
               
               b_kill.setEnabled(false);
               b_scan.setEnabled(true);
               textip.setEditable(true);
            
               InputStream error = process_nmap.getErrorStream();
               error.read(b_streamer);
               String out_err = new String(b_streamer);
               int position0 = out_err.lastIndexOf("QUITTING!");
               
               String control = out_err.substring(0,45);
               String out_err1 = "";
               
               if(control.equalsIgnoreCase("You requested a scan type which requires r00t") || control.equalsIgnoreCase("TCP/IP fingerprinting (for OS scan) requires ") || control.equalsIgnoreCase("Maximum host group size may not be set to les"))
                 {
                  out_err1 = out_err.substring(0, position0-2);  
                 }
               
               else { out_err1 = out_err.substring(0, position0-1); } 
           
               text_er.append("COMMAND LAUNCHED: [ " + cmd + " ]\n");
               text_er.append(out_err1 + "\n\n");
               jmp.setTitle("J[Nmap] - REPORT: Error launching Nmap!");     
              
               labelip.transferFocus();
               textip.setSelectionStart((textip.getText()).length());
              }

           else if(vq == 137) //Uscita forzata.
             {
              Date data = new Date();   
                 
              Tabbed.setEnabled(true);
              iconscan.setIcon(new ImageIcon(getClass().getResource("/img/noscan.gif"))); //Ripristina gif statica.
              status_scan.setText("Scan stopped by user!");
              
              textresult.append("COMMAND LAUNCHED: " + cmd + " stopped by user!");
              textresult.append("\n");
              textresult.append("|--------------------------------------------------------------------------|\n\n");

              b_kill.setEnabled(false);
              b_scan.setEnabled(true);    
              textip.setEditable(true);
                
              String info = timeinfo.getText();
              info = info + " - Stopped: " + data.toString();
              timeinfo.setText(info);
              
              int line = textresult.getLineCount();
              int pos = textresult.getLineStartOffset(line-1);
              textresult.select(pos,pos);
              textresult.setEditable(false);
              jmp.setTitle("J[Nmap] - REPORT: Scan [ " + stat + " ] stopped by user!");
              
              labelip.transferFocus();
              textip.setSelectionStart((textip.getText()).length());
             }
           
           else //Terminazione regolare di nmap.
               {           
                Date data = new Date();   
                   
                InputStream in = process_nmap.getInputStream(); //Restituisce una pipe sull'output del processo.
                
                int offset = 0;
                int fin = leng;
                int leng_copy = leng;
                while(!secure) //I risultati di nmap potrebbero non essere contenibili nell'array corrente.
                  {
                   int read = in.read(b_streamok, offset, fin);
   
                   if(read == leng_copy)
                     {
                      leng = leng * 2;
                      byte[] b_s = new byte[leng];
                      System.arraycopy(b_streamok, 0, b_s, 0, b_streamok.length);
                      b_streamok = b_s;
                      offset = offset + read;
                      fin = b_streamok.length - offset;
                      leng_copy = fin;
                     }
                   
                   else { secure = true; }
                  }

                String output = new String(b_streamok);

                int position = output.lastIndexOf("seconds");
                String out = output.substring(0, position+7);

                Tabbed.setEnabled(true);
                iconscan.setIcon(new ImageIcon(getClass().getResource("/img/noscan.gif"))); //Ripristina gif statica.
                status_scan.setText("REPORT: Scan [ " + stat + " ] finished!");
                textresult = jmp.getTextResult();
                
                textresult.append("COMMAND LAUNCHED: " + cmd);
                textresult.append(out);
                textresult.append("\n");
                textresult.append("|--------------------------------------------------------------------------|\n\n");

                b_kill.setEnabled(false);
                b_scan.setEnabled(true);    
                textip.setEditable(true);
                
                String info = timeinfo.getText();
                info = info + " - Finished: " + data.toString();
                timeinfo.setText(info);
                
                int line = textresult.getLineCount();
                int pos = textresult.getLineStartOffset(line-1);
                textresult.select(pos,pos);
                textresult.setEditable(false);
                jmp.setTitle("J[Nmap] - REPORT: Scan [ " + stat + " ] finished!");
                
                labelip.transferFocus();
                textip.setSelectionStart((textip.getText()).length());
               }
          }
       
       catch(IOException e) { text_er.append("IOException launching: " + cmd); }
       
       catch(InterruptedException oe) { text_er.append("InterruptedException launching: " + cmd); }
       
       catch(BadLocationException eo) {}
      }
  }
